import { csv } from 'd3';
import {
  select,
  json,
  pie,
  arc,
  scaleOrdinal,
  format,
} from 'd3';

const sectionColors = [
  'green',
  'brown',
  'pink',
  'skyblue',
  'grey',
  'violet',
  'orange',
  'wheat',
  'lightgreen',
  'aqua',
]; // Different colors for representation

// Dataset URL made raw from GitHub
const csvUrl =
  'https://gist.githubusercontent.com/Vnai307/ac7ab901f8a484ad779de4aa2ca5e49a/raw/f7d1201ff125e4b7fdc620c41ac457edcd3afb2b/indian-car-sales.csv';

// Fetching data from dataset
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

const xAxisTickFormat = (number) => format('.4~g')(number);

const render = (data, total) => {
  // Sort the data based on Carsales value in descending order
  data.sort((a, b) => b.Carsales - a.Carsales);

  // Select top 5 data points
  const top5Data = data.slice(0, 5);

  const pieData = pie().value((d) => d.Carsales)(top5Data);

  // Define new colors for top 5 data points
  const top5Colors = [
    'red',
    'blue',
    'green',
    'orange',
    'purple',
  ];

  const colors = scaleOrdinal()
    .domain(top5Data.map((d) => d.Company))
    .range(top5Colors);

  const arcGenerator = arc()
    .innerRadius(0)
    .outerRadius(250);

  // Arc
  const segments = arc()
    .innerRadius(0)
    .outerRadius(175)
    .padAngle(0)
    .padRadius(0);

  const sections = svg
    .append('g')
    .attr('transform', `translate(250,250)`)
    .selectAll('path')
    .data(pieData);

  sections
    .enter()
    .append('path')
    .attr('d', segments)
    .attr('fill', (d) => colors(d.data.Company));

  // Section label
  sections
    .enter()
    .append('text')
    .text(
      (d) => xAxisTickFormat(d.data.Carsales / 1000) + 'K',
    )
    .attr('transform', function (d) {
      return 'translate(' + arcGenerator.centroid(d) + ')';
    })
    .style('text-anchor', 'middle')
    .style('fill', 'black')
    .style('font-size', 15);

  // Legends
  const legends = svg
    .append('g')
    .attr('transform', 'translate(500,100)')
    .selectAll('.legends')
    .data(pieData);

  const legend = legends
    .enter()
    .append('g')
    .classed('.legends', true)
    .attr('transform', (d, i) => {
      return `translate(0,${(i + 1) * 30})`;
    });

  legend
    .append('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', (d) => colors(d.data.Company));

  legend
    .append('text')
    .attr('x', 25)
    .attr('y', 15)
    .attr('class', 'legend_text')
    .text((d) => d.data.Company);

  legend
    .append('text')
    .attr('x', 250)
    .attr('y', 15)
    .attr('class', 'legend_value')
    .text(
      (d) => xAxisTickFormat(d.data.Carsales / 1000) + 'K',
    );
};

const run = async () => {
  let data = await fetchData();
  let total = 0;
  data.forEach((d) => {
    d.Carsales = +d.Carsales;
    total += d.Carsales;
  });
  render(data, total);
};
run();
