import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  select,
} from 'd3';

export const scatterPlot = () => {
  let width = 600;
  let height = 400;
  let data = [];
  let xValue = (d) => d.x;
  let yValue = (d) => d.y;
  let margin = { top: 20, right: 20, bottom: 40, left: 40 };
  let radius = 4;
  let svg;

  const my = (selection) => {
    svg = selection
      .append('svg')
      .attr('width', width)
      .attr('height', height);

    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const xAxis = (g) =>
      g
        .attr(
          'transform',
          `translate(0,${height - margin.bottom})`,
        )
        .call(axisBottom(x));

    const yAxis = (g) =>
      g
        .attr('transform', `translate(${margin.left},0)`)
        .call(axisLeft(y));

    svg.append('g').call(xAxis);

    svg.append('g').call(yAxis);

    svg
      .selectAll('.dot')
      .data(data)
      .enter()
      .append('circle')
      .attr('class', 'dot')
      .attr('cx', (d) => x(xValue(d)))
      .attr('cy', (d) => y(yValue(d)))
      .attr('r', radius)
      .style('fill', '#1f77b4');
  };

  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
